(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     12948,        327]
NotebookOptionsPosition[     12516,        308]
NotebookOutlinePosition[     12861,        323]
CellTagsIndexPosition[     12818,        320]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Cookie Roller Generator", "Title",
 CellChangeTimes->{{3.5681212065337286`*^9, 3.5681212426637955`*^9}, {
  3.5692385073558187`*^9, 3.5692385144538307`*^9}}],

Cell["\<\
G. Hart, Jan 2013
Program to input an image file and output a cylinder design for 3D printing, \
to use as a roller that makes a frieze pattern relief on cookies.
See: \[OpenCurlyDoubleQuote]Custom 3D-Printed Rollers for Frieze Pattern \
Cookies\[CloseCurlyDoubleQuote] submitted to Bridges 2013.

1. First create an image file in which degrees of darkness corresponding to \
desired depth.  (White=high cookie surface.  Black=maximum cookie \
indentation.)  The X direction of the image will wrap around the \
circumference of the roller and should be about 500 pixels wide for \
reasonable resolution without an overly large file.
2. Execute the first block of code to select an image file and import it.  \
Verify the displayed image is correct.  
3. Set the desired cylinder height and the alpha parameter, and execute the \
second block of code to generate an image of the cylinder.  The height is in \
arbitrary units which are set in the 3D printer.  Alpha is the fraction of \
the height that black is scaled to, e.g., if alpha=0.1, the deepest indents \
of a 2-inch tall roller will be 0.2 inches.
4. The cylinder model is generated. If it looks OK, execute the final line of \
code to generate the STL file.  (First set the file name to the desired \
output file name.)
5. Send the stl file that is generated to your 3D printer. Note that the \
surface looks like the mirror image of the image file, so that the imprint \
will not be reversed.
\
\>", "Text",
 CellChangeTimes->{{3.568121219616477*^9, 3.5681212380655317`*^9}, {
   3.568121479914365*^9, 3.568121562750103*^9}, {3.5683312989030204`*^9, 
   3.5683316748039794`*^9}, {3.5683317074340253`*^9, 
   3.5683319998688436`*^9}, {3.568332038578898*^9, 3.5683320937489753`*^9}, {
   3.568332168081097*^9, 3.5683321686426983`*^9}, 3.568335309697933*^9, {
   3.5683353815110407`*^9, 3.56833539967508*^9}, {3.5692378038413825`*^9, 
   3.569237894695942*^9}, {3.5692379377052174`*^9, 3.569237985410102*^9}, {
   3.569238533423464*^9, 3.5692385523774977`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetDirectory", "[", "$UserDocumentsDirectory", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"img", "=", 
  RowBox[{"ColorConvert", "[", 
   RowBox[{
    RowBox[{"Import", "[", 
     RowBox[{"SystemDialogInput", "[", "\"\<FileOpen\>\"", "]"}], "]"}], ",", 
    "\"\<Grayscale\>\""}], "]"}], "      ", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
     RowBox[{
     "import", " ", "the", " ", "image", " ", "as", " ", "values", " ", "in", 
      " ", "range", " ", "0"}], ".."}], "1"}], " ", "*)"}]}]}], "Input",
 CellChangeTimes->{{3.568121183513412*^9, 3.568121188708709*^9}, {
   3.568121327107625*^9, 3.5681213499329305`*^9}, {3.568121995393849*^9, 
   3.5681220267546425`*^9}, {3.5681228797944336`*^9, 
   3.5681228802514596`*^9}, {3.5681278237882137`*^9, 3.568127827604432*^9}, {
   3.568128053453287*^9, 3.5681280536092873`*^9}, {3.568303909816568*^9, 
   3.5683039420066133`*^9}, 3.5683098817226677`*^9, 3.5683323933078213`*^9, {
   3.5684870559608736`*^9, 3.5684870606036806`*^9}, {3.5684873035360374`*^9, 
   3.568487309727646*^9}, {3.5686519100773473`*^9, 3.568651934850191*^9}, 
   3.5692362205166016`*^9, {3.569238193935668*^9, 3.5692382393317475`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"height", " ", "=", " ", "2.0"}], ";"}], "   ", 
  RowBox[{"(*", " ", 
   RowBox[{"height", " ", "of", " ", 
    RowBox[{"cylinder", ".", "  ", "Units"}], " ", "are", " ", "set", " ", 
    "in", " ", "the", " ", "3", "D", " ", "printer"}], " ", "*)"}]}], "\n", 
 RowBox[{
  RowBox[{"alpha", " ", "=", " ", "0.1"}], ";", "   ", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"max", " ", "relief", " ", "depth"}], ",", " ", 
    RowBox[{"as", " ", "fraction", " ", "of", " ", "height"}], ",", " ", 
    RowBox[{"typically", " ", "around", " ", "0.1"}], ",", " ", 
    RowBox[{"or", " ", "-", 
     RowBox[{"0.1", " ", "for", " ", "inverting"}]}]}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"n", ",", "m"}], "}"}], " ", "=", " ", 
   RowBox[{"ImageDimensions", "[", "img", "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"rMid", "=", 
   RowBox[{"n", " ", "*", " ", 
    RowBox[{"height", "/", 
     RowBox[{"(", 
      RowBox[{"2", " ", "Pi", " ", "m"}], ")"}]}]}]}], " ", ";", "     ", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
    "scale", " ", "so", " ", "pixels", " ", "are", " ", "square", " ", "at", 
     " ", "mid"}], "-", "depth"}], "*)"}], "\n", 
  RowBox[{"r", "=", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"rMid", "-", 
       RowBox[{"alpha", " ", "height", " ", 
        RowBox[{"(", 
         RowBox[{"#", "-", "0.5"}], ")"}]}]}], "&"}], ",", 
     RowBox[{"ImageData", "[", "img", "]"}], ",", 
     RowBox[{"{", "2", "}"}]}], "]"}]}], ";", "   ", 
  RowBox[{"(*", " ", 
   RowBox[{"convert", " ", "to", " ", "table", " ", "of", " ", "radii"}], " ",
    "*)"}], "\[IndentingNewLine]", 
  RowBox[{"xyz", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"r", "[", 
         RowBox[{"[", 
          RowBox[{"j", ",", "i"}], "]"}], "]"}], 
        RowBox[{"Sin", "[", 
         RowBox[{"2", " ", "Pi", " ", 
          RowBox[{"i", "/", "n"}]}], "]"}]}], ",", 
       RowBox[{
        RowBox[{"r", "[", 
         RowBox[{"[", 
          RowBox[{"j", ",", "i"}], "]"}], "]"}], 
        RowBox[{"Cos", "[", 
         RowBox[{"2", " ", "Pi", " ", 
          RowBox[{"i", "/", "n"}]}], "]"}]}], ",", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", 
          RowBox[{"j", "/", "m"}]}], ")"}], " ", "height"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", "n"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "1", ",", "m"}], "}"}]}], "]"}]}], ";", "   ", 
  RowBox[{"(*", " ", 
   RowBox[{"convert", " ", "to", " ", "Cartesian"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
   "the", " ", "cylinder", " ", "consists", " ", "of", " ", "three", " ", 
    RowBox[{"parts", ":", " ", 
     RowBox[{
     "the", " ", "curved", " ", "surface", " ", "with", " ", "the", " ", 
      "image", " ", "and", " ", "two", " ", "circular", " ", 
      RowBox[{"caps", ":"}]}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{"surface", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"xyz", "[", 
        RowBox[{"[", 
         RowBox[{"i", ",", "j"}], "]"}], "]"}], ",", 
       RowBox[{"xyz", "[", 
        RowBox[{"[", 
         RowBox[{
          RowBox[{"Mod", "[", 
           RowBox[{
            RowBox[{"i", "+", "1"}], ",", "n", ",", "1"}], "]"}], ",", "j"}], 
         "]"}], "]"}], ",", 
       RowBox[{"xyz", "[", 
        RowBox[{"[", 
         RowBox[{
          RowBox[{"Mod", "[", 
           RowBox[{
            RowBox[{"i", "+", "1"}], ",", "n", ",", "1"}], "]"}], ",", 
          RowBox[{"j", "+", "1"}]}], "]"}], "]"}], ",", 
       RowBox[{"xyz", "[", 
        RowBox[{"[", 
         RowBox[{"i", ",", 
          RowBox[{"j", "+", "1"}]}], "]"}], "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", "n"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "1", ",", 
       RowBox[{"m", "-", "1"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"bottom", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
       RowBox[{"xyz", "[", 
        RowBox[{"[", 
         RowBox[{"i", ",", "m"}], "]"}], "]"}], ",", 
       RowBox[{"xyz", "[", 
        RowBox[{"[", 
         RowBox[{
          RowBox[{"Mod", "[", 
           RowBox[{
            RowBox[{"i", "+", "1"}], ",", "n", ",", "1"}], "]"}], ",", "m"}], 
         "]"}], "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", "n"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"top", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "0", ",", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", "-", 
            RowBox[{"1", "/", "m"}]}], ")"}], "height"}]}], "}"}], ",", 
       RowBox[{"xyz", "[", 
        RowBox[{"[", 
         RowBox[{
          RowBox[{"Mod", "[", 
           RowBox[{
            RowBox[{"i", "+", "1"}], ",", "n", ",", "1"}], "]"}], ",", "1"}], 
         "]"}], "]"}], ",", 
       RowBox[{"xyz", "[", 
        RowBox[{"[", 
         RowBox[{"i", ",", "1"}], "]"}], "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", "n"}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{"g", "=", 
  RowBox[{"Graphics3D", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"EdgeForm", "[", "]"}], ",", 
     RowBox[{"Map", "[", 
      RowBox[{"Polygon", ",", 
       RowBox[{"Join", "[", 
        RowBox[{"surface", ",", "top", ",", "bottom"}], "]"}]}], "]"}]}], 
    "}"}], "]"}], "   ", 
  RowBox[{"(*", " ", 
   RowBox[{
   "generate", " ", "image", " ", "to", " ", "see", " ", "if", " ", "alpha", 
    " ", "looks", " ", "OK"}], " ", "*)"}]}]}], "Input",
 CellChangeTimes->{{3.56812135569726*^9, 3.568121466976625*^9}, {
   3.5681215735777225`*^9, 3.5681216133589973`*^9}, 3.5681220288197603`*^9, {
   3.568122291218769*^9, 3.5681222914737835`*^9}, {3.5681235169208755`*^9, 
   3.5681235170778837`*^9}, 3.568123616029544*^9, {3.5681255524132986`*^9, 
   3.5681256327718945`*^9}, {3.5681334329786606`*^9, 3.568133433128661*^9}, 
   3.5683039888810835`*^9, 3.568309552811391*^9, {3.568309619750686*^9, 
   3.5683096213906884`*^9}, {3.5683097914145317`*^9, 
   3.5683098053545513`*^9}, {3.568330440047179*^9, 3.568330441960782*^9}, {
   3.5683321845031247`*^9, 3.5683321866035275`*^9}, {3.568332228738388*^9, 
   3.568332304500095*^9}, 3.5683323662877836`*^9, {3.5683324241030684`*^9, 
   3.568332479238752*^9}, {3.568334996355011*^9, 3.568335045794839*^9}, {
   3.5683351072193527`*^9, 3.5683351762332993`*^9}, 3.5684873976877775`*^9, {
   3.568487635580527*^9, 3.568487645200141*^9}, {3.568487699765421*^9, 
   3.568487703723427*^9}, {3.5684878071127796`*^9, 3.5684878129355884`*^9}, {
   3.568489397784934*^9, 3.568489419635766*^9}, 3.5686516815681458`*^9, {
   3.5686518292848053`*^9, 3.568651841203226*^9}, 3.5686518844465027`*^9, {
   3.569023912833048*^9, 3.569023916487053*^9}, 3.5692360031146197`*^9, {
   3.5692371568770466`*^9, 3.569237164146659*^9}, {3.569237249463209*^9, 
   3.569237249759609*^9}, {3.5692375099056664`*^9, 3.5692375444597273`*^9}, 
   3.5692375907606087`*^9, {3.5692377590069036`*^9, 3.5692377619397087`*^9}, 
   3.5692380534730215`*^9, {3.569238101021905*^9, 3.569238130427956*^9}, 
   3.5692383672675724`*^9, {3.5692384243948727`*^9, 3.5692384312276845`*^9}, {
   3.569238664463694*^9, 3.569238672762909*^9}, 3.5692387700446796`*^9, 
   3.5692388081523466`*^9, {3.5692388440168095`*^9, 3.5692388545468283`*^9}, 
   3.56923902085972*^9, 3.569239099828059*^9, 3.5692394612338943`*^9, {
   3.569373062069963*^9, 3.5693730783251915`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"SystemDialogInput", "[", 
     RowBox[{"\"\<FileSave\>\"", ",", "\"\<roller.stl\>\""}], "]"}], ",", 
    "g"}], "]"}], "    ", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"If", " ", "the", " ", "relief", " ", "looks", " ", "good"}], ",",
     " ", 
    RowBox[{"create", " ", "stl", " ", 
     RowBox[{"file", "."}]}]}], " ", "*)"}]}]], "Input",
 CellChangeTimes->{{3.569236887121372*^9, 3.5692368935017834`*^9}, {
  3.5692370343232307`*^9, 3.5692370383168383`*^9}, {3.569238435330492*^9, 
  3.569238471272955*^9}, {3.5692385889907618`*^9, 3.5692385925007677`*^9}}]
}, Open  ]]
},
WindowSize->{1712, 817},
WindowMargins->{{-10, Automatic}, {Automatic, 36}},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (October 6, 2011)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 163, 2, 83, "Title"],
Cell[745, 26, 2029, 34, 191, "Text"],
Cell[2777, 62, 1208, 24, 52, "Input"],
Cell[3988, 88, 7866, 200, 232, "Input"],
Cell[11857, 290, 643, 15, 31, "Input"]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
